<?php
namespace App\Models;
use Illuminate\Database\Eloquent\Model;

class RechargeApi extends Model {

    private $rechargeUrl;

    function __construct() {

        $this->rechargeUrl = env('RECHARGE_URL', true);
        $this->rechargeApiKey = env('RECHARGE_API_KEY',true);
    }
   
    public function getChargeById($id) {

        $method = 'GET';
        $path = $this->rechargeUrl . 'charges/'.$id;
        $response = $this->call($method, $path);
        return json_decode($response);

    }

    public function getDiscountCode($code) {

        $method = 'GET';
        $path = $this->rechargeUrl . 'discounts?discount_code='.$code;
        $response = $this->call($method, $path);
        return json_decode($response);

    }
    
    /**
     * apply discount to a charge by discount code
     */
    function applyDiscountToChargeByDiscountCode($chargeID, $data) {

        $method = 'POST';
        $path = $this->rechargeUrl .'charges/'.$chargeID.'/apply_discount';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }


    /**
     * Change charge schedule date
     */
    function updateChargeScheduleDate($chargeID, $data) {

        $method = 'POST';
        $path = $this->rechargeUrl .'charges/'.$chargeID.'/change_next_charge_date';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }
    
    /**
     * change subscription next charge date
     */
    function updateSubscriptionChargeDate($subscriptionID, $data) {

        $method = 'POST';
        $path = $this->rechargeUrl.'subscriptions/'.$subscriptionID.'/set_next_charge_date';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get charge details by subscriptionID
     */
    function getChargeDetailsBySubscriptionID($subscriptionID) {

        $method = 'GET';
        $path = $this->rechargeUrl.'charges?subscription_id='.$subscriptionID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }
   

    /**
     * get subscription by subscription_id
     */
    function getSubscription($subscription_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . 'subscriptions/'.$subscription_id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer details by shopify customer Id
     */
    function getCustomerDetailsByShopifyCustomerId($customerId) {

        $method = 'GET';
        $path = $this->rechargeUrl . 'customers?shopify_customer_id='.$customerId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * CURL call
     */
    function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . env('RECHARGE_API_KEY',true),
            "content-type: application/json",
        ));
     
        $response = curl_exec($curl);
        return $response;

    }
}
